################################################################################
#
#  build_uboot_config.sh
#
#  Copyright (c) 2019-2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
################################################################################

TARGET_DEVICE=goldfish
TARGET_BASE=sav526
UBOOT_SUBPATH="bootable/bootloader/uboot/sigmastar/5xx"
UBOOT_DEFCONFIG_NAME="cocoa_${TARGET_BASE}_defconfig"

# Expected image files are seperated with ":"
UBOOT_IMAGES="u-boot_spinand.img.bin"

TARGET_ARCH="arm"
TOOLCHAIN_REPO="https://releases.linaro.org/components/toolchain/binaries/7.2-2017.11/arm-linux-gnueabihf/gcc-linaro-7.2.1-2017.11-x86_64_arm-linux-gnueabihf.tar.xz"
TOOLCHAIN_NAME="gcc-linaro-7.2.1-2017.11-x86_64_arm-linux-gnueabihf"
TOOLCHAIN_PREFIX="arm-linux-gnueabihf-"

UBOOT_EXTRA_CFLAGS="-DCOCOA_KERNEL_SIZE=0x00800000 -DCOCOA_RTOS_SIZE=0x00200000 -DCOCOA_UBI_MTD=17 -DCOCOA_RECOVERY_KERNEL_PART_NAME=KERNEL_P -DCOCOA_RECOVERY_BACKUP_KERNEL_PART_NAME=KERNEL_M -DCOCOA_RTOS_PART_NAME=RTOS_M -DCOCOA_RTOS_BACKUP_PART_NAME=RTOS_P -DCOCOA_RTOS_BOOT_ADDRESS=0x27808000 -DCOCOA_RECOVERY_KERNEL_LOAD_ADDRESS=0x21000000 -DCOCOA_USERDATA_SIZE=33554432 -DCOCOA_DATA_PARTITION=customer -DCOCOA_BOOTCONFIG_SIZE=131072 -DCOCOA_RECOVERY_SIZE=7340032 -DCOCOA_BLUE_SIGNATURE_VERIFICATION_PKCS=1 -DCOCOA_BLUE_SIGNATURE_VERIFICATION_PSS=0 -DCOCOA_KERNEL_DISABLE_COMMS=\"ipc.disable_rtos=1 amazon_sdio.disable_comms=1\""

function platform_prepare_uboot_build {
    mkdir -p ${UBOOT_SUBPATH}/include
    echo "/* AUTOGENERATED HEADER FILE - DO NOT MODIFY MANUALLY */" > ${UBOOT_SUBPATH}/include/autogen_defines.h
    echo "#define COCOA_MTDPARTS_DEFAULT \"mtdparts=nand0:0x20000@0x140000(IPL0),0x20000(IPL1),0x20000(IPL2),0x20000(IPL3),0x20000(IPL4),0x20000(IPL5),0x40000(IPL_CUST0),0x40000(IPL_CUST1),0x100000(MISC),0xc0000(UBOOT0),0xc0000(UBOOT1),0x40000(ENV),0x200000(RTOS_P),0x200000(RTOS_M),0x800000(KERNEL_P),0x800000(KERNEL_M),0x2000000(ROOTFS),0x4680000(UBI),\"" >> ${UBOOT_SUBPATH}/include/autogen_defines.h
    echo "#define COCOA_PRODUCT_NAME \"${TARGET_DEVICE}\"" >> ${UBOOT_SUBPATH}/include/autogen_defines.h
}
